# License: GNU AGPL v3: http://www.gnu.org/licenses/
# This file is part of `delarte` (https://git.afpy.org/fcode/delarte.git)

"""Provide data model types."""


from typing import NamedTuple, Optional


#
# Metadata objects
#
class Program(NamedTuple):
    """A program metadata."""

    id: str
    language: str
    title: str
    subtitle: str


class Rendition(NamedTuple):
    """A program rendition metadata."""

    class Audio(NamedTuple):
        """An audio rendition."""

        language: str
        original: bool
        visual_impaired: bool

    class Subtitles(NamedTuple):
        """A subtitles rendition."""

        language: str
        hearing_impaired: bool

    label: str
    audio: Audio
    subtitles: Optional[Subtitles]
    protocol: str


class Variant(NamedTuple):
    """A program variant metadata."""

    class Video(NamedTuple):
        """A video rendition."""

        resolution: tuple[int, int]
        frame_rate: float

    average_bandwidth: int
    video: Video


#
# Track objects
#
class VideoTrack(NamedTuple):
    """A video track."""

    width: int
    height: int
    frame_rate: float


class AudioTrack(NamedTuple):
    """An audio track."""

    name: str
    language: str
    original: bool
    visual_impaired: bool


class SubtitlesTrack(NamedTuple):
    """A subtitles track."""

    name: str
    language: str
    hearing_impaired: bool


#
# Source objects
#
class ProgramSource(NamedTuple):
    """A program source."""

    program: Program
    player_config_url: str


class RenditionSource(NamedTuple):
    """A rendition source."""

    program: Program
    rendition: Rendition
    program_index_url: str


class VariantSource(NamedTuple):
    """A variant source."""

    program: Program
    rendition: Rendition
    variant: Variant
    video_index_url: str
    audio_index_url: str
    subtitles_index_url: Optional[str]


class Target(NamedTuple):
    """A download target item."""

    class VideoInput(NamedTuple):
        """A video input."""

        track: VideoTrack
        url: str

    class AudioInput(NamedTuple):
        """An audio input."""

        track: AudioTrack
        url: str

    class SubtitlesInput(NamedTuple):
        """A subtitles input."""

        track: SubtitlesTrack
        url: str

    video_input: VideoInput
    audio_input: AudioInput
    subtitles_input: Optional[SubtitlesInput]
    title: str | tuple[str, str]
    output: str
