# License: GNU AGPL v3: http://www.gnu.org/licenses/
# This file is part of `delarte` (https://git.afpy.org/fcode/delarte.git)

"""Provide ArteTV website utilities."""

import json

from .error import (
    HTTPError,
    InvalidPage,
    InvalidPageData,
    PageNotFound,
    PageNotSupported,
)

_DATA_MARK = '<script id="__NEXT_DATA__" type="application/json">'


def fetch_page_content(http, page_url):
    """Fetch html content at given URL."""
    r = http.request("GET", page_url)

    # special handling of 404
    if r.status == 404:
        raise PageNotFound()
    HTTPError.raise_for_status(r)

    return r.data.decode("utf-8")


def extract_page_data(html_content):
    """Extract JSON page data from html content."""
    start = html_content.find(_DATA_MARK)
    if start < 0:
        raise InvalidPage("DATA_MARK_NOT_FOUND")
    start += len(_DATA_MARK)
    end = html_content.index("</script>", start)

    try:
        return json.loads(html_content[start:end].strip())
    except json.JSONDecodeError:
        raise InvalidPage("INVALID_JSON_DATA")


def _find_zones(page_value, code_prefix):
    return [
        zone for zone in page_value["zones"] if zone["code"].startswith(code_prefix)
    ]


def _find_unique_zone(page_value, code_prefix):
    zones = _find_zones(page_value, code_prefix)
    if len(zones) != 1:
        raise InvalidPageData("ZONE_COUNT")
    return zones[0]


def _find_items(zone, item_type):
    return [item for item in zone["content"]["data"] if item["type"] == item_type]


def _find_unique_item(zone, item_type):
    items = _find_items(zone, item_type)
    if len(items) != 1:
        raise InvalidPageData("ITEM_COUNT")
    return items[0]


def _read_program_page(page_value):
    language = page_value["language"]

    zone = _find_unique_zone(page_value, "program_content_")
    item = _find_unique_item(zone, "program")
    return (
        item["programId"],
        language,
        item["title"],
        item["subtitle"],
    )


def _read_collection_page(page_value):
    language = page_value["language"]

    main_zone = _find_unique_zone(page_value, "collection_videos_")
    main_items = _find_items(main_zone, "teaser")

    sub_zones = _find_zones(page_value, "collection_subcollection_")

    if sub_zones:
        if main_items:
            raise InvalidPageData("MIXED_ZONES")
        items = [
            item for sub_zone in sub_zones for item in _find_items(sub_zone, "teaser")
        ]
    elif main_items:
        items = main_items
    else:
        return []

    return [
        (
            item["programId"],
            language,
            item["title"],
            item["subtitle"],
        )
        for item in items
    ]


def read_page_data(page_data):
    """Return programs listed on given JSON page data."""
    try:
        page_value = page_data["props"]["pageProps"]["props"]["page"]["value"]

        match page_value["type"]:
            case "program":
                return [_read_program_page(page_value)]
            case "collection":
                return _read_collection_page(page_value)
            case _:
                raise PageNotSupported()

    except (KeyError, IndexError, ValueError) as e:
        raise InvalidPage("SCHEMA") from e
